package com.example.gallery.ui.photos

import android.os.Bundle
import android.view.LayoutInflater
import android.view.View
import android.view.ViewGroup
import androidx.core.view.isVisible
import androidx.fragment.app.Fragment
import androidx.fragment.app.viewModels
import androidx.navigation.fragment.findNavController
import androidx.navigation.fragment.navArgs
import androidx.recyclerview.widget.StaggeredGridLayoutManager
import com.example.gallery.databinding.FragmentPhotosBinding
import com.google.android.material.snackbar.Snackbar

class PhotosFragment : Fragment() {
    private var _binding: FragmentPhotosBinding? = null
    private val binding get() = _binding!!

    private val viewModel: PhotosViewModel by viewModels()
    private lateinit var photoAdapter: PhotoAdapter
    private val args: PhotosFragmentArgs by navArgs()

    override fun onCreateView(
        inflater: LayoutInflater,
        container: ViewGroup?,
        savedInstanceState: Bundle?
    ): View {
        _binding = FragmentPhotosBinding.inflate(inflater, container, false)
        return binding.root
    }

    override fun onViewCreated(view: View, savedInstanceState: Bundle?) {
        super.onViewCreated(view, savedInstanceState)

        setupRecyclerView()
        setupSwipeRefresh()
        observeViewModel()

        viewModel.loadPhotos(args.album)
    }

    private fun setupRecyclerView() {
        photoAdapter = PhotoAdapter { photo ->
            val action = PhotosFragmentDirections.actionPhotosFragmentToPhotoDetailFragment(photo)
            findNavController().navigate(action)
        }

        binding.recyclerView.apply {
            adapter = photoAdapter
            layoutManager = StaggeredGridLayoutManager(2, StaggeredGridLayoutManager.VERTICAL)
            setHasFixedSize(true)
        }
    }

    private fun setupSwipeRefresh() {
        binding.swipeRefreshLayout.setOnRefreshListener {
            viewModel.refreshPhotos(args.album.id)
        }
    }

    private fun observeViewModel() {
        viewModel.photos.observe(viewLifecycleOwner) { photos ->
            photoAdapter.submitList(photos)
            binding.emptyView.isVisible = photos.isEmpty()
        }

        viewModel.isLoading.observe(viewLifecycleOwner) { isLoading ->
            binding.swipeRefreshLayout.isRefreshing = isLoading
            binding.shimmerLayout.isVisible = isLoading && photoAdapter.itemCount == 0

            if (isLoading) {
                binding.shimmerLayout.startShimmer()
            } else {
                binding.shimmerLayout.stopShimmer()
            }
        }

        viewModel.error.observe(viewLifecycleOwner) { errorMessage ->
            if (!errorMessage.isNullOrEmpty()) {
                Snackbar.make(binding.root, errorMessage, Snackbar.LENGTH_LONG)
                    .setAction("Повторить") {
                        viewModel.refreshPhotos(args.album.id)
                    }
                    .show()
            }
        }

        viewModel.isFromCache.observe(viewLifecycleOwner) { isFromCache ->
            if (isFromCache) {
                Snackbar.make(binding.root, "Данные загружены из кэша. Нет подключения к интернету.", Snackbar.LENGTH_LONG).show()
            }
        }

        viewModel.albumTitle.observe(viewLifecycleOwner) { title ->
            requireActivity().title = title
        }
    }

    override fun onDestroyView() {
        super.onDestroyView()
        _binding = null
    }
}