package com.example.gallery.ui.photos

import android.content.Context
import android.net.ConnectivityManager
import android.net.NetworkCapabilities
import androidx.lifecycle.LiveData
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.example.gallery.App
import com.example.gallery.models.Album
import com.example.gallery.models.Photo
import com.example.gallery.repository.GalleryRepository
import kotlinx.coroutines.launch

class PhotosViewModel : ViewModel() {
    private val repository = GalleryRepository()

    private val _photos = MutableLiveData<List<Photo>>()
    val photos: LiveData<List<Photo>> = _photos

    private val _isLoading = MutableLiveData<Boolean>()
    val isLoading: LiveData<Boolean> = _isLoading

    private val _error = MutableLiveData<String?>()
    val error: LiveData<String?> = _error

    private val _albumTitle = MutableLiveData<String>()
    val albumTitle: LiveData<String> = _albumTitle

    private val _isFromCache = MutableLiveData<Boolean>()
    val isFromCache: LiveData<Boolean> = _isFromCache

    fun loadPhotos(album: Album) {
        _albumTitle.value = album.title
        _isFromCache.value = false

        album.photos?.let {
            _photos.value = it
            return
        }

        viewModelScope.launch {
            _isLoading.value = true
            _error.value = null

            repository.getAlbumDetails(album.id).fold(
                onSuccess = { albumDetails ->
                    _photos.value = albumDetails.photos ?: emptyList()
                    _isLoading.value = false

                    if ((albumDetails.photos?.isNotEmpty() == true) && !isInternetAvailable()) {
                        _isFromCache.value = true
                    }
                },
                onFailure = { e ->
                    _error.value = e.message ?: "Неизвестная ошибка"
                    _isLoading.value = false
                }
            )
        }
    }

    fun refreshPhotos(albumId: Int) {
        viewModelScope.launch {
            _isLoading.value = true
            _error.value = null
            _isFromCache.value = false

            repository.getAlbumDetails(albumId).fold(
                onSuccess = { albumDetails ->
                    _photos.value = albumDetails.photos ?: emptyList()
                    _isLoading.value = false

                    if ((albumDetails.photos?.isNotEmpty() == true) && !isInternetAvailable()) {
                        _isFromCache.value = true
                    }
                },
                onFailure = { e ->
                    _error.value = e.message ?: "Неизвестная ошибка"
                    _isLoading.value = false
                }
            )
        }
    }

    // Проверка доступности интернета
    private fun isInternetAvailable(): Boolean {
        val connectivityManager = App.instance.getSystemService(Context.CONNECTIVITY_SERVICE) as ConnectivityManager
        val network = connectivityManager.activeNetwork ?: return false
        val networkCapabilities = connectivityManager.getNetworkCapabilities(network) ?: return false
        return networkCapabilities.hasCapability(NetworkCapabilities.NET_CAPABILITY_INTERNET)
    }
}